package wesen.robot;

import java.io.*;
import wesen.util.ObjectCloner;

/**
 * Ist ein Motor eines Wesens (links oder rechts)
 *
 *
 * @author Wesen-Team
 * @version 1.0
 * @since 30.08.2001
 */
public class Motor implements Serializable {

  /**
   * Faktor fuer die Umrechnung der normierten Geschwindigkeit (0..1)
   * auf Pixel-Geschwindigkeit umgerechnet
   * (wenn zu hoch gewhlt -> Sensoren werden zu langsam berechnet und 
   * hinken nach)
   */  
  final static int  SPEED_FACTOR = 8;
  /*
  final static int  SPEED_FACTOR = 3; //Mike Penti II MMX, 266  
  */
  
  /**
   * Grundgeschwindigkeit, zwischen 0 - 1. Public weil XML-Attribute
   */
  public double     v;

  /**
   * Maximale Geschwindigkeit, zwischen 0 - 1. Public weil XML-Attribute
   */
  public double     vMax;

  /**
   * Minimale Geschwindigkeit, zwischen 0 - 1. //todo: XML
   */
  public double     vMin;

  /**
   * Beschleunigung
   * (Wird nicht bentzt. Gedacht war zuerst, dass der Motor<br>
   * indirekt via dia Beschleunigung stimuliert werden soll.)
   */
  double            a = 0 * SPEED_FACTOR;
  
  /**
   * Maximale Beschleunigung
   * (Wird nicht bentzt. Gedacht war zuerst, dass der Motor<br>
   * indirekt via dia Beschleunigung stimuliert werden soll.)
   */
  double            aMax = 1 * SPEED_FACTOR;
  
  /** Verschiebung des Motors der letzen deltaTimeInMillis */
  double            deltaX;   
  
  /** Verschiebung des Motors der letzen deltaTimeInMillis */
  double            deltaY;   

  /** Drehung des Motors um den anderen Motor als Drehpunkt in den letzen deltaTimeInMillis in Radian */
  double            deltaAngle;

  /**
   * Konstruktor fuer den Motor wird fuer das Anlegen ueber XML gebraucht.
   */
  public Motor() {}

  /**
   * Klonen dieses Objektes und rueckgabe der Referenz auf dieses neue Objekt.
   * 
   * @return Das erzeugte Objekt
   */
  public Motor cloneObject() throws Exception {
    return (Motor) (ObjectCloner.deepCopy(this));
  } 

  /**
   * Berechnet die neue Verschiebung des Motors 
   *
   * @param radius
   * @param deltaTimeInMillis
   * @param startangle
   */
  public void CalcNewDeltas(double radius, long deltaTimeInMillis, 
                            double startangle) {

    
    double  strecke = 0;

    strecke = (((double) deltaTimeInMillis) / 100) * v * SPEED_FACTOR;

    // System.out.println("Time: " + deltaTimeInMillis);
    // System.out.println("Strecke: " + strecke);
    // System.out.println("V: " + v);
    double  gamma, beta;

    gamma = 2 * Math.asin(strecke / (2 * radius));
    beta = (Math.PI - gamma) / 2;
    deltaX = strecke * Math.cos(startangle + beta - Math.PI / 2);

    // Minus wegen Bildschirmorientierung (0,0) oben links
    deltaY = -strecke * Math.sin(startangle + beta - Math.PI / 2);
    deltaAngle = gamma;   // =gamma?
  } 
  
  /**
   * Setzt die neuberechnete Geschwindigkeit<br> 
   * und hlt sie in den Grenzen vMin und vMax.
   * 
   * @param newV Geschwindigkeit
   */
  public void setV(double newV){
    if (newV > vMax) v = vMax;
    else if (newV < vMin) v=vMin;
    else v=newV;
  }

}

