package wesen.robot;

import wesen.simulator.*;
import wesen.util.*;

/**
 * Diese Klasse reprsentiert einen Lichtsensor.
 * Er kann nur von Lichtquellen gereizt werden.
 *
 *
 * @author Wesen-Team
 * @version 1.0
 * @since
 */
public class SensorLicht extends Sensor implements SensorInterface {

  
   
  /**
   * Konstruktor fuer den SensorLicht wird fuer das Anlegen ueber XML gebraucht.
   */
  public SensorLicht() {}

  /**
   * Konstruktor
   *
   * @param Siehe Erklrung bei den gleichnamigen Instanzvariablen
   *
   */
  public SensorLicht(boolean blnIsRight, double range, Roboter rob, 
                     char motorFactorLeft, char motorFactorRight) {
    super(blnIsRight, range, rob, motorFactorLeft, motorFactorRight);
  }

  /**
   * Passt den Sensorwert an aufgrund der im Range stehenden Lichtquellen
   * @param ov - Enthlt Roboter und andere Aktoren
   * @param index - Position von this.rob innerhalb des ov
   */
  public void updateSensorValue(ObjectVector ov, int index) {
    double  val = 0;
    double  distance = 0;
    double chi=0; //Hilfswinkel Sensor->Aktor
    Actor   a;

    // Loope ber alle Aktoren
    for (int i = 0; i < ov.size(); i++) {

      // Position des aktuellen Aktors (Lichtquelle) holen
      // (Aktor index (dieser Roboter) ausschliessen)
      Object  obj = ov.elementAt(i);

      // Test
      // if(obj instanceof Actor) System.out.println(" " + ((Actor)obj).getHasLight() );
      // System.out.println(i + " " + (obj instanceof Actor) );//+ " " + ((Actor)obj).getHasLight());
      if (i != index && obj instanceof Actor && ((Actor) obj).getHasLight()) {
        a = (Actor) ov.elementAt(i);

        // Distanz zwischen Sensor und Aktorenrand
        distance = Geometrie.getDistance(posX, posY, a.getX(), a.getY());

        // Berechnung von Chi        
        chi = this.getChi(a);
               
        //Debug
        if (rob.getDebugSichtwinkel() &&  Simulator.withDebug){
          if (blnIsRight) System.out.println("Rechter Sensor"); else System.out.println("Linker Sensor");
          System.out.println("Winkel Sensor-Aktor: " + Math.toDegrees(chi));
          System.out.println("Sichtfeldgrenzwinkel links vom Sensor  (> chi): " + Math.toDegrees(rob.alpha + viewAngle / 2));
          System.out.println("Sichtfeldgrenzwinkel rechts vom Sensor (< chi): " + Math.toDegrees(rob.alpha - viewAngle / 2));
          if(blnIsRight) System.out.println();
        }
        
        // Prfe ob Sensor den Aktor sehen kann
        double dblMaxSichtWinkel = (rob.alpha + viewAngle / 2) % (2 * Math.PI); //Normiert
        double dblMinSichtWinkel = (rob.alpha - viewAngle / 2) % (2 * Math.PI); //Normiert
        
        if ( dblMaxSichtWinkel > chi && dblMinSichtWinkel < chi) {

          // Sensor ist sichtbar
          // Val addieren
          // Achtung: Knnte rel. gross werden
          val = val + 1 - distance / range;
          
          //debug
          if (rob.getDebugValueDistanceRange() && Simulator.withDebug){
            if (blnIsRight) System.out.println("Rechter Sensor:" + val);
            else System.out.println("Linker Sensor: " + val);
            System.out.println("Distance=" + distance + ", Range=" + range + "\n");  
          }
          
        }   // if
       }    // if
     }      // for i
 
    // Ergebnis auf Bereich 0->1 zuschneiden
    if (val < 0) {
      val = 0;
    } else if (val > 1) {
      val = 1;

      // Return
    } 

    this.setNewValue(val);

    // Test
    // if (test) System.out.println("Val: " + val +"\n");
  } 
  
  /**
   * Berechnet den Hilfswinkel chi zwischen dem Sensor und dem Aktor
   * 
   * Info: Es gibt 4 Normalflle wie ein Sensor und Aktor sich gegenber stehen knnen.<br>
   * Dokumentiert im Draft von Mike.<b>
   *
   * @return 0 <= chi <= 2*Pi im Gegenuhrzeigersinn ab Position Ost   
   */
  private double getChi(Actor actor){
    double chi=0;
    
    //Spezialflle (Verhindern u.a. Division durch 0)
    if (this.posX == actor.getX()){
      if (this.posY > actor.getY()) chi = Math.PI/2;
      else if (this.posY > actor.getY()) chi = -Math.PI/2;
      else chi=0;
    }
    else if (this.posY == actor.getX()){
      if (this.posX < actor.getX()) chi = Math.PI;
      else if (this.posX > actor.getX()) chi=0;
      else chi=0; //wird eh schon oben agbefangen
    }  
    else{
      //Normalflle
      double dblGK = (double)this.posY - (double)actor.getY();
      double dblAK = (double)actor.getX() - (double)this.posX;
      double dblVerh =  (double)dblGK / dblAK ;
      chi=Math.atan( dblVerh ); //Fall 1

      //Fall 3
      if (this.posX < actor.getX() && this.posY < actor.getY())
        chi = chi + 2*Math.PI;
      
      //Fall 2 und 4 (Korrektur von chi NEU)
      else if (this.posX > actor.getX()) {        
        chi = chi + Math.PI;
      }
      
      //Debug
      if (rob.getDebugChiRight() && blnIsRight && Simulator.withDebug){
        System.out.println("Normalfall Chi:");
        System.out.print("S = ( " + this.posX + " , " + this.posY + " )");
        System.out.println(" A = ( " + actor.getX() + " , " + actor.getY() + " )");
        System.out.println("Verhltnis GK/AK= " + dblGK + " / " + dblAK + " = " + dblVerh);
        System.out.println("Chi in Radian = " + chi );
        System.out.println("Chi in Grad = " + Math.toDegrees(chi) + "\n");        
        System.out.println("Richtung Roboter = " + Math.toDegrees(rob.alpha));
      }      
      
    }
    return chi;    
  }
  
  
  
 
}

