package wesen.simulator;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.awt.image.BufferedImage;
import java.util.*;
import java.lang.ClassLoader;
import java.net.URL;


/**
 * Dies ist das Panel auf welchen sich die Aktoren befinden bzw. die Roboter bewegen.
 * Alle Benutzerinteraktionen auf dieses Panel werden in diesem Objekt behandelt. 
 *
 * @author Wesen-Team
 * @version 1.0
 */
public class Umgebung extends JPanel implements MouseListener, 
        MouseMotionListener {

  /**
   * Damit kann erzwungen werden, dass die Umgbung neu aufgebaut wird. >0 Die Umgebung wird geloescht
   */
  private int             flag = 0;


  /**
   * Referenz auf das Simulate-Objekt
   * @supplierCardinality 1
   */
  private Simulate        simulate;

  /**
   * Enthalt das aktuelle markierte Objekt. -1= kein Objekt ist markiert
   */
  private int             workId = -1;

  /**
   * Vector mit den XML (Roboter) und Properties (Aktoren) - Dateinamen (ohne Pfad)
   */
  private Vector          vcFileName = new Vector();

  /**
   * Popup Menu fuer die Auswahl eines Objektes
   */
  private JPopupMenu      popup;

  /**
   * Dieses Image weleches das Panel ueberlagert
   */
  private BufferedImage   offScreenImage;

  /**
   * Zum BufferedImage, ist dies der entsprechenden Grafikkontext
   */
  private Graphics2D      offscreenImageGraphics;

  /**
   * Dieser Vektor enthaelt alle Objekte der Umgebung
   * @supplierCardinality 1
   */
  private AnimationVector av;

  /**
   * Position des Mauszeigers in der X-Achse
   */
  private double          dX;

  /**
   * Position des Mauszeigers in der Y-Achse
   */
  private double          dY;
  
  
  

  
  /**
   * Referenz auf das Simulator-Objekt
   * @supplierCardinality 1
   */ 
  private Simulator       simulator;

  /**
   * @supplierCardinality 0..1 
   */
  /*#  private SuffixFileFilter lnkSuffixFileFilter; */

  /**
   * Erstellt ein Objekt und registriert sich bei der Simulate-Klasse
   * 
   * @param frame Referenz auf das Haupt-Frame
   */
  Umgebung(Simulator simulator) {
    this.simulator = simulator;
    // Das folgende ToolTip wird nie angezeit, aber es wird benoetigt das die hier
    // ueberschriebene Methode getToolTipText(MouseEvent) zum Zuge kommt.
    setToolTipText("Simulationsumgebung");
    addMouseListener(this);
    addMouseMotionListener(this);

    simulate = Simulate.getInstance(simulator, this);
  }

  /**
   * Erstellt ein Popupmenu mit den Dateinamen und definiert die entsprechende Aktion auf die Wahl
   * eines Menu Eintrages
   */
  public void init() {
    popup = new JPopupMenu();

    ActionListener  al = new ActionListener() {

      /**
       * Es wurde ein Objekt aus dem Popupmenu gewaehlt
       * 
       * @param e Event
       */
      public void actionPerformed(ActionEvent e) {
        String  strItem = e.getActionCommand();
        int     i = 0;

        // Suche den gesamten Dateinamen
        while (i < vcFileName.size() 
               &&!((String) vcFileName.elementAt(i)).startsWith(strItem)) {
          i++;
        }   // while
 
        av.addObject((int) dX, (int) dY, (String) vcFileName.elementAt(i));
        repaint();
      }     // actionPerformed
 
    }; // ActionListener
    JMenuItem item;

    // Erstellt das Popupmenu fuer die Objekte
    for (int l = 0; l < 2; l++) {

      // l == 0 : Roboternamen werden eingelesen
      // l == 1 : Aktorennamen werden eingelesen
      String  fileSuffix = (l == 0) ? av.getRobotSuffix() : av.getActorSuffix();
      File    entries[] = getRobotFiles(fileSuffix);
      int     suffixL = fileSuffix.length();

      for (int i = 0; i < entries.length; i++) {
        if (!entries[i].isDirectory()) {
          String  fullName = entries[i].getName();

          vcFileName.addElement(fullName);
          popup.add(item = new JMenuItem(fullName.substring(0, 
                  fullName.length() - suffixL)));
          item.addActionListener(al);
        }   // if
       }    // for
 
      if (l == 0) {
        popup.add(new JSeparator());
      }     // if
     }      // for
 
    this.add(popup);
  }   // init
 
  /**
   * Die Maustaste wurde gedrueckt
   * 
   * @param e Event
   * @see java.awt.event.MouseListener#mousePressed(MouseEvent)
   */
  public void mousePressed(MouseEvent e) {

    /* System.out.println("mousePressed"); */
    if (!simulate.isSimRunning()) {
      if (!checkPopup(e)) {
        workId = av.getObjectAtPointID((int) dX, (int) dY);
      }   // if
     }    // if
   }      // mousePressed
 
  /**
   * Es erfolgte eine Mausclick
   * 
   * @param e Event
   * @see java.awt.event.MouseListener#mouseClicked(MouseEvent)
   */
  public void mouseClicked(MouseEvent e) {
    /* System.out.println("mouseClicked"); */
    if (!simulate.isSimRunning()) {
      if (!checkPopup(e)) {
        if (e.getClickCount() == 2 && Simulator.withDebug) {
          av.startBeanEditor((int) dX, (int) dY, this);
        }
        // Markieren eines Objektes
        if ((workId = av.getObjectAtPointID((int) dX, (int) dY)) >= 0) {
          av.setMark(simulator, workId, true);
        } else {
          av.setMark(simulator, workId, false);
        } // else

        repaint();
      } // if
    }   // if

  } // mouseClicked 

  /**
   * Die Maustaste wurde freigelassen
   * 
   * @param e Event
   * @see java.awt.event.MouseListener#mouseReleased(MouseEvent)
   */
  public void mouseReleased(MouseEvent e) {

    /* System.out.println("mouseReleased"); */
    if (!simulate.isSimRunning()) {
      checkPopup(e);
    } // if
  }   // mouseReleased
 
  /**
   * Die Maus betritt die Umgebung
   * 
   * @param e Event
   * @see java.awt.event.MouseListener#mouseEntered(MouseEvent)
   */
  public void mouseEntered(MouseEvent e) {}

  /**
   * Die Maus verlaesst die Umgebung
   * 
   * @param e Event
   * @see java.awt.event.MouseListener#mouseExited(MouseEvent)
   */
  public void mouseExited(MouseEvent e){
    /* System.out.println("mouseExited"); */

  }

  /**
   * Ein Objekt wird neu platziert
   *
   * @param e Event
   *
   * @see java.awt.event.MouseMotionListener#mouseDragged(Event)
   */
  public void mouseDragged(MouseEvent e) {

    /* System.out.println("mouseDragged"); */
    if (workId >= 0) {
      dX = e.getX();
      dY = e.getY();
      int radius = av.getObjectDiameter(workId) / 2;
      if(dX - radius > 0 && dX + radius < this.getWidth() 
        && dY - radius > 0 && dY + radius < this.getHeight()) {
        av.setPos(workId, (int) dX, (int) dY);
        repaint();
      }  // if
    }   // if
  } // mouseDragged


  /**
   * Setzt den ToolTip entsprechend der Mausposition fuer einen Roboter, Aktoren
   * oder nichts
   * @param e Event
   * @return the text of the tool tip
   * @see javax.swing.JComponent#mouseMovedgetToolTipText(MouseEvent)
   */
   public String getToolTipText(MouseEvent e) {
     Point p = e.getPoint();
     return av.getToolTipOfObjectAtPoint((int) p.getX(), (int) p.getY());
   }

  /**
   * Die Maus wurde verschoben
   *
   * @param e Event
   *
   * @see java.awt.event.MouseMotionListener#mouseMoved(Event)
   */
  public void mouseMoved(MouseEvent e) {

    /* System.out.println("mouseMoved"); */
  } 

  /**
   * Falls die entsprechende Mousetaste gedrueckt wurde, wird ein Popup-Menu angezeigt
   * 
   * @param e Event
   * 
   * @return true wenn ein Objekt aus dem Popup gewaehlt wurde, andernfalls false
   */
  private boolean checkPopup(MouseEvent e) {
    dX = e.getX();
    dY = e.getY();

    /* int iModi = e.getModifiers(); */
    if (e.isPopupTrigger()) {
      popup.show(e.getComponent(), e.getX(), e.getY());

      workId = -1;

      return true;
    }   // if
 
    return false;
  }   // checkPopup
 
  /**
   * Speichert eine Referenz auf den Animationsvektor
   * 
   * @param av Referenz auf den AnimationsVektor
   */
  public void setAnimationVector(AnimationVector av) {
    this.av = av;
  } 

  /**
   * Rueckgabe aller Dateinamen eines Verzeichnisses welche dem Filter entsprechen.
   * 
   * @param Filter fuer die Endung des Dateinamen
   * @return Array mit den Dateinamen welche dem Filter entsprechen
   */
  private File[] getRobotFiles(String fileSuffix) {
    File[]      entries = null;
    FileFilter  fileFilter = new SuffixFileFilter(fileSuffix);
    File        dir = new File(av.getDirOfRobotData());
    // Verzeichniseintrge lesen
    entries = dir.listFiles(fileFilter);

    return entries;
  } 

  /**
   * Setzen des markierten Objektes im Vektor
   * 
   * @param value -1 = Kein Objekt ist markiert; >0 Das entsprechende Objekt ist markiert.
   */
  public void setWorkId(int value) {
    workId = value;
  } 

  /**
   * Beeinflusst das Verhalten ob die Simulationsumgebung neu aufgebaut wird.
   * @param value >0 Simulations neu aufbauen
   */
  public void setFlag(int value) {
    flag = value;
  } 

  /**
   * Die Umgebung wird neu aufgebaut. Die Bewegungspur wird geloescht.
   */
  public void forceRepaint() {
    setFlag(1);
    repaint();
  } 

    


  public void update(Graphics g) {
    paintComponent(g);
  } 

  /**
   * Ueberschreibt die paintComponent Methode
   * 
   * @param g Grafikkontext
   * 
   * @see javax.swing.JComponent#paintComponent(Graphics)
   */
  public void paintComponent(Graphics g) {
    Graphics2D  graphics2d = (Graphics2D) g;

    super.paintComponent(graphics2d);
    checkOffscreenImage();


    if (!simulator.tbTrails.isSelected() || workId >= 0 || flag > 0) {
      // Das Bild wird geloescht
      offscreenImageGraphics.clearRect(0, 0, getWidth(), getHeight());
      setFlag(0);
    } 

    av.drawIt(offscreenImageGraphics);

    // Fuer eine AWT wuerde folgendes gebraucht
    // paint(offscreenImageGraphics);
    graphics2d.drawImage(offScreenImage, null, 0, 0);
  }   // paintComponent
 
  /**
   * Erzeugt ein neues BufferedImage, welches als Spielflaeche benutzt wird
   */
  public void checkOffscreenImage() {

    System.out.println("checkOffscreenImage");
    if (offScreenImage == null 

    /*
     * || frame.getWidth() != frameWidth
     * || frame.getHeight() != frameHeight)
     */
    || offScreenImage.getWidth() - getWidth() > 2 || offScreenImage.getWidth() 
            - getWidth() < 0 || offScreenImage.getHeight() 
                           - getHeight() > 2 || offScreenImage.getHeight() 
                                           - getHeight() < 0) {

      /*
       * frameWidth = frame.getWidth();
       * frameHeight = frame.getHeight();
       */
      offScreenImage = new BufferedImage(getWidth(), getHeight(), 
                                         BufferedImage.TYPE_INT_ARGB);

      // In das offscreenImageGrphics koennen wir zeichnen
      offscreenImageGraphics = offScreenImage.createGraphics();

      offscreenImageGraphics.setBackground(getBackground());
    }   // if
   }    // checkOffscreenImage
 
}       // Umgebung

/**
 * Ein Dateinamen-Filter welcher auf die Dateierweiterung selektioniert
 * 
 * 
 * @author Hugo Graf
 * @version 1.0
 * @since 04.07.2001
 */
class SuffixFileFilter implements FileFilter {
  private String  fSuffix;

  /**
   * Die Methode gibt true zurueck falls der Dateiname dem Filter entspricht, andernfalls wird
   * false zurueckgegeben.
   *
   * @param file Dateiname
   * 
   * @return true - Dateiname ist OK; false Dateiname entspricht nicht dem Filter
   * 
   * @see javax.swing.filechooser.FileFilter#accept(File)
   */
  public boolean accept(File file) {
    if (file == null || fSuffix == null) {
      return false;
    } // if

    return file.getName().endsWith(fSuffix);
  }

  /**
   * Ein Filter nach der Dateinamenendung wird angelegt.
   * 
   * @param suffix Dateinamenendung
   * 
   * @see javax.swing.filechooser.FileFilter
   */
  public SuffixFileFilter(String suffix) {
    fSuffix = suffix;
  }
}   // SuffixFileFilter

